// DummyNote.cs created with MonoDevelop
// User: calvin at 10:56 AM 2/12/2008
//
// To change standard headers go to Edit->Preferences->Coding->Standard Headers
//

using System;
using Tasque;

namespace Tasque.Backends.Dummy
{
	public class DummyNote : INote
	{
		public string Name
		{
			get { return ""; }
			set { // TODO: Implement something 
			}
		}
   
		public string Text
		{
			get { return ""; }
			set { // TODO: Implement something 
			}
		}

	}
}